import urlparse

import requests

from thredds_crawler.crawl import Crawl

class ThreddsCollector(object):
    def __init__(self, url, selects=None, skips=None):
        self.selects    = selects
        self.skips      = skips
        self.url        = url

    def run(self):
        c = Crawl(self.url, self.selects, self.skips)
        urls = [s.get("url") for d in c.datasets for s in d.services if s.get("service").lower() == "iso"]
        return urls



import os
import sys
import unittest

from metadown.collectors.geonetwork import GeoNetworkCollector

from metadown.downloader import XmlDownloader

class GeoNetworkTest(unittest.TestCase):
    scratch_dir = os.path.join(os.path.dirname(__file__),'scratch')
    isos = GeoNetworkCollector("http://data.glos.us/metadata").run()
    XmlDownloader.run(isos, scratch_dir, namer=GeoNetworkCollector.namer, modifier=GeoNetworkCollector.modifier)


try:
    from lxml import etree
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as etree
    except ImportError:
        try:
            # Python 2.5
            import xml.etree.ElementTree as etree
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as etree
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as etree
                except ImportError:
                    raise RuntimeError('You need either lxml or ElementTree')


import os
import sys
import unittest

from metadown.collectors.thredds import ThreddsCollector

from metadown.downloader import XmlDownloader

class ThreddsTest(unittest.TestCase):

    def setUp(self):
        self.scratch = os.path.join(os.path.dirname(__file__),'scratch')

    def test_thredds_custom_skips(self):
        # We only want the Agg datasets
        selects = [".*SST-Agg"]
        # Don't process the "files/" lists
        skips = ["files/"]
        isos = ThreddsCollector("http://tds.glos.us:8080/thredds/mtri/aoc.html", selects=selects, skips=skips).run()
        # 5 Lake ISOs
        print isos
        assert len(isos) == 5
        XmlDownloader.run(isos, self.scratch)
        assert os.path.isfile(os.path.join(self.scratch, "LakeErieSST-Agg.xml"))
        assert os.path.isfile(os.path.join(self.scratch, "LakeHuronSST-Agg.xml"))
        assert os.path.isfile(os.path.join(self.scratch, "LakeMichiganSST-Agg.xml"))
        assert os.path.isfile(os.path.join(self.scratch, "LakeSuperiorSST-Agg.xml"))
        assert os.path.isfile(os.path.join(self.scratch, "LakeOntarioSST-Agg.xml"))

    def test_thredds_default_skips(self):
        # We only want the Agg and Latest
        selects = [".*Nowcast-Agg", ".*Lastest-Forecast"]
        isos = ThreddsCollector("http://tds.glos.us:8080/thredds/hecwfs/hecwfs.html", selects=selects).run()
        # 2 ISOs (Nowcast and latest Forecast)
        assert len(isos) == 2
        XmlDownloader.run(isos, self.scratch)
        assert os.path.isfile(os.path.join(self.scratch, "HECWFS-Latest-Forecast.nc.xml"))
        assert os.path.isfile(os.path.join(self.scratch, "HECWFS-Nowcast-Agg.nc.xml"))



import os
import csv
import tempfile
import codecs
from urlparse import urlsplit
from shutil import abspath

import requests

from metadown.utils.etree import etree

class GeoNetworkCollector(object):
    def __init__(self, base_url):
        self.data = base_url + '/srv/en/csv.search?'
        self.download = base_url + '/srv/en/iso19139.xml?id='

    def utf_8_encoder(self, unicode_csv_data):
        for line in unicode_csv_data:
            yield line.encode('utf-8')

    def run(self):

        isos = []

        o, t =  tempfile.mkstemp()
        with codecs.open(t, "w+", "utf-8") as h:
            h.write(requests.get(self.data).text)

        with codecs.open(t, "rb", "utf-8") as f:
            reader = csv.DictReader(self.utf_8_encoder(f), delimiter='\t')
            for row in reader:
                if row.get('schema') != 'iso19139':
                    continue

                download_url = self.download + row.get('id')
                isos.append(download_url)

        os.unlink(f.name)

        return isos

    @staticmethod
    def namer(url, **kwargs):
        uid = urlsplit(url).query
        uid = uid[uid.index("=")+1:]
        return "GeoNetwork-" + uid + ".xml"

    @staticmethod
    def modifier(url, **kwargs):
        # translate ISO19139 to ISO19115
        gmi_ns = "http://www.isotc211.org/2005/gmi"
        etree.register_namespace("gmi",gmi_ns)

        new_root = etree.Element("{%s}MI_Metadata" % gmi_ns)
        old_root = etree.parse(url).getroot()

        # carry over an attributes we need
        [new_root.set(k,v) for k,v in old_root.attrib.items()]
        # carry over children
        [new_root.append(e) for e in old_root]

        return etree.tostring(new_root, encoding="UTF-8", pretty_print=True, xml_declaration=True)
                


import requests

from metadown.utils.etree import etree

class InSituCollector(object):
    def __init__(self, url, category):
        self.url = url
        self.category = category

    def run(self):
        isos = []

        resp = requests.get(self.url + "/sources.html").text
        tree = etree.HTML(resp)

        for source in tree.findall(".//li"):
            if source.text.lower() == self.category.lower():

                # get list of files in source
                dirlist = requests.get(self.url + "/" + source.text + '/list.html').text
                dirtree = etree.HTML(dirlist)

                for iso in dirtree.findall('.//li'):
                    isos.append(self.url + "/" + source.text + '/' + iso.text.strip())

        return isos



import os
from urlparse import urlsplit

import requests

class XmlDownloader(object):

    @classmethod
    def run(cls, url_list, download_path, namer=None, modifier=None, **kwargs):
        for url in url_list:

            if namer is not None:
                # Custom naming function for saved file on disk
                filename = namer(url, **kwargs)
            else:
                # By default, get the local file from the remove name name
                filename = os.path.basename(urlsplit(url).path)
            
            # Always save with a .xml extension
            if os.path.splitext(filename)[1] != ".xml":
                filename = filename + ".xml"

            # Absolute path to save file
            filepath = os.path.join(download_path, filename)

            
            if modifier is not None:
                data = modifier(url, **kwargs)
            else:
                try:
                    data = requests.get(url).text
                except requests.exceptions.RequestException:
                    print "Error downloading %s" % url
                    continue
                
            # Need to use codecs.open for UTF-8 data
            with open(filepath, "w") as handle:
                handle.write(data)



import urlparse

def construct_url(root, url, path):

    if path[0] == "/":
        # Absolute paths
        cat = root + path
    elif path[0:4] == "http":
        # Full HTTP links
        cat = path
    else:
        # Relative paths.  Strip off last filename
        url = "/".join(urlparse.urlsplit(url).path.split("/")[0:-1])
        cat = root + url + "/" + path

    return cat



from __future__ import with_statement
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

from metadown import __version__

def readme():
    with open('README.md') as f:
        return f.read()

reqs = [line.strip() for line in open('requirements.txt')]

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)

setup(
    name                = "metadown",
    version             = __version__,
    description         = "A Python library for collecting Met/Ocean metadata from various services",
    long_description    = readme(),
    license             = 'GPLv3',
    author              = "Kyle Wilcox",
    author_email        = "kwilcox@sasascience.com",
    url                 = "https://github.com/asascience-open/metadown",
    packages            = find_packages(),
    install_requires    = reqs,
    tests_require       = ['pytest'],
    cmdclass            = {'test': PyTest},
    classifiers         = [
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python',
            'Topic :: Scientific/Engineering',
        ],
    include_package_data = True,
) 
